import { PirschClientConfig, PirschHit, PirschDomain, PirschFilter, PirschKeyword, PirschScreenClassStats, PirschPlatformStats, PirschCountryStats, PirschBrowserStats, PirschOSStats, PirschReferrerStats, PirschLanguageStats, PirschVisitorHourStats, PirschActiveVisitorsData, PirschGrowth, PirschConversionGoal, PirschEventStats, PirschPageStats, PirschVisitorStats, PirschUTMTermStats, PirschUTMContentStats, PirschUTMCampaignStats, PirschUTMMediumStats, PirschUTMSourceStats, PirschTimeSpentStats, PirschTotalVisitorStats, PirschEventListStats, PirschOSVersionStats, PirschBrowserVersionStats, PirschEntryStats, PirschExitStats, PirschCityStats, PirschHttpOptions, PirschAccessMode, Scalar, Optional, PirschSession, PirschBatchHit, PirschBatchSession, TagStats } from "./types";
import { PirschApiError, PirschCommon } from "./common";
export declare abstract class PirschCoreClient extends PirschCommon {
    protected readonly version = "v1";
    protected readonly endpoint = "api";
    protected readonly clientId?: string;
    protected readonly clientSecret?: string;
    protected readonly baseUrl: string;
    protected readonly timeout: number;
    protected readonly accessMode: PirschAccessMode;
    protected accessToken: string;
    /**
     * The constructor creates a new client.
     *
     * @param {object} configuration You need to pass in the **Client ID** and **Client Secret** or **Access Key** you have configured on the Pirsch dashboard.
     * It's also recommended to set the proper protocol for your website, else it will be set to `https` by default.
     * All other configuration parameters can be left to their defaults.
     * @param {string} configuration.baseUrl The base URL for the pirsch API
     * @param {number} configuration.timeout The default HTTP timeout in milliseconds
     * @param {string} configuration.clientId The OAuth client ID
     * @param {string} configuration.clientSecret The OAuth client secret
     * @param {string} configuration.protocol The default HTTP protocol to use for tracking
     *
     */
    constructor(configuration: PirschClientConfig);
    /**
     * hit sends a hit to Pirsch. Make sure you call it in all request handlers you want to track.
     * Also, make sure to filter out unwanted pathnames (like /favicon.ico in your root handler for example).
     *
     * @param hit all required data for the request.
     * @returns APIError or an empty promise, in case something went wrong
     */
    hit(hit: PirschHit): Promise<Optional<PirschApiError>>;
    /**
     * batchHit sends batched hits to Pirsch.
     *
     * @param hits all required data for the request.
     * @returns APIError or an empty promise, in case something went wrong
     */
    batchHits(hits: PirschBatchHit[]): Promise<Optional<PirschApiError>>;
    /**
     * event sends an event to Pirsch. Make sure you call it in all request handlers you want to track.
     * Also, make sure to filter out unwanted pathnames (like /favicon.ico in your root handler for example).
     *
     * @param name the name for the event
     * @param hit all required data for the request
     * @param duration optional duration for the event
     * @param meta optional object containing metadata (only scalar values, like strings, numbers, and booleans)
     * @returns APIError or an empty promise, in case something went wrong
     */
    event(name: string, hit: PirschHit, duration?: number, meta?: Record<string, Scalar>): Promise<Optional<PirschApiError>>;
    /**
     * batchEvents sends batched events to Pirsch.
     *
     * @param events all required data for the request.
     * @returns APIError or an empty promise, in case something went wrong
     */
    batchEvents(events: {
        name: string;
        hit: PirschHit;
        time: string;
        duration?: number;
        meta?: Record<string, Scalar>;
    }[]): Promise<Optional<PirschApiError>>;
    /**
     * session keeps a session alive.
     *
     * @param session all required data for the request.
     * @returns APIError or an empty promise, in case something went wrong
     */
    session(session: PirschSession): Promise<Optional<PirschApiError>>;
    /**
     * batchSessions keeps batched sessions alive.
     *
     * @param sessions all required data for the request.
     * @returns APIError or an empty promise, in case something went wrong
     */
    batchSessions(sessions: PirschBatchSession[]): Promise<Optional<PirschApiError>>;
    /**
     * domain returns the domain for this client.
     *
     * @returns Domain object for this client.
     */
    domain(): Promise<PirschDomain | PirschApiError>;
    /**
     * sessionDuration returns the session duration grouped by day.
     *
     * @param filter used to filter the result set.
     */
    sessionDuration(filter: PirschFilter): Promise<PirschTimeSpentStats[] | PirschApiError>;
    /**
     * timeOnPage returns the time spent on pages.
     *
     * @param filter used to filter the result set.
     */
    timeOnPage(filter: PirschFilter): Promise<PirschTimeSpentStats[] | PirschApiError>;
    /**
     * utmSource returns the utm sources.
     *
     * @param filter used to filter the result set.
     */
    utmSource(filter: PirschFilter): Promise<PirschUTMSourceStats[] | PirschApiError>;
    /**
     * utmMedium returns the utm medium.
     *
     * @param filter used to filter the result set.
     */
    utmMedium(filter: PirschFilter): Promise<PirschUTMMediumStats[] | PirschApiError>;
    /**
     * utmCampaign returns the utm campaigns.
     *
     * @param filter used to filter the result set.
     */
    utmCampaign(filter: PirschFilter): Promise<PirschUTMCampaignStats[] | PirschApiError>;
    /**
     * utmContent returns the utm content.
     *
     * @param filter used to filter the result set.
     */
    utmContent(filter: PirschFilter): Promise<PirschUTMContentStats[] | PirschApiError>;
    /**
     * utmTerm returns the utm term.
     *
     * @param filter used to filter the result set.
     */
    utmTerm(filter: PirschFilter): Promise<PirschUTMTermStats[] | PirschApiError>;
    /**
     * totalVisitors returns the total visitor statistics.
     *
     * @param filter used to filter the result set.
     */
    totalVisitors(filter: PirschFilter): Promise<PirschTotalVisitorStats | PirschApiError>;
    /**
     * visitors returns the visitor statistics grouped by day.
     *
     * @param filter used to filter the result set.
     */
    visitors(filter: PirschFilter): Promise<PirschVisitorStats[] | PirschApiError>;
    /**
     * entryPages returns the entry page statistics grouped by page.
     *
     * @param filter used to filter the result set.
     */
    entryPages(filter: PirschFilter): Promise<PirschEntryStats[] | PirschApiError>;
    /**
     * exitPages returns the exit page statistics grouped by page.
     *
     * @param filter used to filter the result set.
     */
    exitPages(filter: PirschFilter): Promise<PirschExitStats[] | PirschApiError>;
    /**
     * pages returns the page statistics grouped by page.
     *
     * @param filter used to filter the result set.
     */
    pages(filter: PirschFilter): Promise<PirschPageStats[] | PirschApiError>;
    /**
     * conversionGoals returns all conversion goals.
     *
     * @param filter used to filter the result set.
     */
    conversionGoals(filter: PirschFilter): Promise<PirschConversionGoal[] | PirschApiError>;
    /**
     * events returns all events.
     *
     * @param filter used to filter the result set.
     */
    events(filter: PirschFilter): Promise<PirschEventStats[] | PirschApiError>;
    /**
     * eventMetadata returns the metadata for a single event.
     * The event name and metadata key must be set in the filter, or otherwise no results will be returned.
     *
     * @param filter used to filter the result set.
     */
    eventMetadata(filter: PirschFilter): Promise<PirschEventStats[] | PirschApiError>;
    /**
     * listEvents returns a list of all events including metadata.
     *
     * @param filter used to filter the result set.
     */
    listEvents(filter: PirschFilter): Promise<PirschEventListStats[] | PirschApiError>;
    /**
     * eventPages returns all pages an event has been triggered on.
     * The event name must be set in the filter, or otherwise no results will be returned.
     *
     * @param filter used to filter the result set.
     */
    eventPages(filter: PirschFilter): Promise<PirschPageStats[] | PirschApiError>;
    /**
     * growth returns the growth rates for visitors, bounces, ...
     *
     * @param filter used to filter the result set.
     */
    growth(filter: PirschFilter): Promise<PirschGrowth | PirschApiError>;
    /**
     * activeVisitors returns the active visitors and what pages they're on.
     *
     * @param filter used to filter the result set.
     */
    activeVisitors(filter: PirschFilter): Promise<PirschActiveVisitorsData | PirschApiError>;
    /**
     * timeOfDay returns the number of unique visitors grouped by time of day.
     *
     * @param filter used to filter the result set.
     */
    timeOfDay(filter: PirschFilter): Promise<PirschVisitorHourStats[] | PirschApiError>;
    /**
     * languages returns language statistics.
     *
     * @param filter used to filter the result set.
     */
    languages(filter: PirschFilter): Promise<PirschLanguageStats[] | PirschApiError>;
    /**
     * referrer returns referrer statistics.
     *
     * @param filter used to filter the result set.
     */
    referrer(filter: PirschFilter): Promise<PirschReferrerStats[] | PirschApiError>;
    /**
     * os returns operating system statistics.
     *
     * @param filter used to filter the result set.
     */
    os(filter: PirschFilter): Promise<PirschOSStats[] | PirschApiError>;
    /**
     * osVersions returns operating system version statistics.
     *
     * @param filter used to filter the result set.
     */
    osVersions(filter: PirschFilter): Promise<PirschOSVersionStats[] | PirschApiError>;
    /**
     * browser returns browser statistics.
     *
     * @param filter used to filter the result set.
     */
    browser(filter: PirschFilter): Promise<PirschBrowserStats[] | PirschApiError>;
    /**
     * browserVersions returns browser version statistics.
     *
     * @param filter used to filter the result set.
     */
    browserVersions(filter: PirschFilter): Promise<PirschBrowserVersionStats[] | PirschApiError>;
    /**
     * country returns country statistics.
     *
     * @param filter used to filter the result set.
     */
    country(filter: PirschFilter): Promise<PirschCountryStats[] | PirschApiError>;
    /**
     * city returns city statistics.
     *
     * @param filter used to filter the result set.
     */
    city(filter: PirschFilter): Promise<PirschCityStats[] | PirschApiError>;
    /**
     * platform returns the platforms used by visitors.
     *
     * @param filter used to filter the result set.
     */
    platform(filter: PirschFilter): Promise<PirschPlatformStats[] | PirschApiError>;
    /**
     * screen returns the screen classes used by visitors.
     *
     * @param filter used to filter the result set.
     */
    screen(filter: PirschFilter): Promise<PirschScreenClassStats[] | PirschApiError>;
    /**
     * screen returns the screen classes used by visitors.
     *
     * @param filter used to filter the result set.
     */
    tagKeys(filter: PirschFilter): Promise<TagStats[] | PirschApiError>;
    /**
     * screen returns the screen classes used by visitors.
     *
     * @param filter used to filter the result set.
     */
    tags(filter: PirschFilter): Promise<TagStats[] | PirschApiError>;
    /**
     * keywords returns the Google keywords, rank, and CTR.
     *
     * @param filter used to filter the result set.
     */
    keywords(filter: PirschFilter): Promise<PirschKeyword[] | PirschApiError>;
    private performPost;
    private performGet;
    private performFilteredGet;
    private getFilterParams;
    private refreshToken;
    private generateUrl;
    private accessModeCheck;
    protected abstract post<Response, Data extends object = object>(url: string, data: Data, options?: PirschHttpOptions): Promise<Response>;
    protected abstract get<Response>(url: string, options?: PirschHttpOptions): Promise<Response>;
    protected abstract toApiError(error: unknown): Promise<PirschApiError>;
}
