"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PirschCoreClient = void 0;
const constants_1 = require("./constants");
const common_1 = require("./common");
class PirschCoreClient extends common_1.PirschCommon {
    /**
     * The constructor creates a new client.
     *
     * @param {object} configuration You need to pass in the **Client ID** and **Client Secret** or **Access Key** you have configured on the Pirsch dashboard.
     * It's also recommended to set the proper protocol for your website, else it will be set to `https` by default.
     * All other configuration parameters can be left to their defaults.
     * @param {string} configuration.baseUrl The base URL for the pirsch API
     * @param {number} configuration.timeout The default HTTP timeout in milliseconds
     * @param {string} configuration.clientId The OAuth client ID
     * @param {string} configuration.clientSecret The OAuth client secret
     * @param {string} configuration.protocol The default HTTP protocol to use for tracking
     *
     */
    constructor(configuration) {
        super();
        this.version = "v1";
        this.endpoint = "api";
        this.accessToken = "";
        const { baseUrl = constants_1.PIRSCH_DEFAULT_BASE_URL, timeout = constants_1.PIRSCH_DEFAULT_TIMEOUT } = configuration;
        this.baseUrl = baseUrl;
        this.timeout = timeout;
        if ("accessToken" in configuration) {
            const { accessToken } = configuration;
            this.assertAccessTokenCredentials({ accessToken });
            this.accessToken = accessToken;
            this.accessMode = "access-token";
        }
        else if ("clientId" in configuration || "clientSecret" in configuration) {
            const { clientId, clientSecret } = configuration;
            this.assertOauthCredentials({ clientId, clientSecret });
            this.clientId = clientId;
            this.clientSecret = clientSecret;
            this.accessMode = "oauth";
        }
        else {
            throw new Error(`Missing credentials, please supply either '${JSON.stringify({
                clientId: "value",
                clientSecret: "value",
            })}' or  '${JSON.stringify({
                accessToken: "value",
            })}'!`);
        }
    }
    /**
     * hit sends a hit to Pirsch. Make sure you call it in all request handlers you want to track.
     * Also, make sure to filter out unwanted pathnames (like /favicon.ico in your root handler for example).
     *
     * @param hit all required data for the request.
     * @returns APIError or an empty promise, in case something went wrong
     */
    hit(hit) {
        return __awaiter(this, void 0, void 0, function* () {
            if (hit.dnt === "1") {
                return;
            }
            return yield this.performPost(constants_1.PirschEndpoint.HIT, hit);
        });
    }
    /**
     * batchHit sends batched hits to Pirsch.
     *
     * @param hits all required data for the request.
     * @returns APIError or an empty promise, in case something went wrong
     */
    batchHits(hits) {
        return __awaiter(this, void 0, void 0, function* () {
            const filtered = hits.filter(hit => hit.dnt !== "1");
            if (filtered.length === 0) {
                return;
            }
            return yield this.performPost(constants_1.PirschEndpoint.HIT_BATCH, filtered);
        });
    }
    /**
     * event sends an event to Pirsch. Make sure you call it in all request handlers you want to track.
     * Also, make sure to filter out unwanted pathnames (like /favicon.ico in your root handler for example).
     *
     * @param name the name for the event
     * @param hit all required data for the request
     * @param duration optional duration for the event
     * @param meta optional object containing metadata (only scalar values, like strings, numbers, and booleans)
     * @returns APIError or an empty promise, in case something went wrong
     */
    event(name, hit, duration = 0, meta) {
        return __awaiter(this, void 0, void 0, function* () {
            if (hit.dnt === "1") {
                return;
            }
            const event = Object.assign({ event_name: name, event_duration: duration, event_meta: this.prepareScalarObject(meta) }, hit);
            return yield this.performPost(constants_1.PirschEndpoint.EVENT, event);
        });
    }
    /**
     * batchEvents sends batched events to Pirsch.
     *
     * @param events all required data for the request.
     * @returns APIError or an empty promise, in case something went wrong
     */
    batchEvents(events) {
        return __awaiter(this, void 0, void 0, function* () {
            const filtered = events.filter(event => event.hit.dnt !== "1");
            if (filtered.length === 0) {
                return;
            }
            const results = filtered.map(({ name, hit, time, duration = 0, meta }) => {
                const event = Object.assign({ event_name: name, event_duration: duration, event_meta: this.prepareScalarObject(meta), time }, hit);
                return event;
            });
            return yield this.performPost(constants_1.PirschEndpoint.EVENT_BATCH, results);
        });
    }
    /**
     * session keeps a session alive.
     *
     * @param session all required data for the request.
     * @returns APIError or an empty promise, in case something went wrong
     */
    session(session) {
        return __awaiter(this, void 0, void 0, function* () {
            if (session.dnt === "1") {
                return;
            }
            return yield this.performPost(constants_1.PirschEndpoint.SESSION, session);
        });
    }
    /**
     * batchSessions keeps batched sessions alive.
     *
     * @param sessions all required data for the request.
     * @returns APIError or an empty promise, in case something went wrong
     */
    batchSessions(sessions) {
        return __awaiter(this, void 0, void 0, function* () {
            const filtered = sessions.filter(session => session.dnt !== "1");
            if (filtered.length === 0) {
                return;
            }
            return yield this.performPost(constants_1.PirschEndpoint.SESSION_BATCH, filtered);
        });
    }
    /**
     * domain returns the domain for this client.
     *
     * @returns Domain object for this client.
     */
    domain() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("domain");
            const result = yield this.performGet(constants_1.PirschEndpoint.DOMAIN);
            const error = new common_1.PirschDomainNotFoundApiError();
            if (Array.isArray(result) && result.length === 0) {
                return error;
            }
            else if (Array.isArray(result)) {
                return (_a = result.at(0)) !== null && _a !== void 0 ? _a : error;
            }
            return result;
        });
    }
    /**
     * sessionDuration returns the session duration grouped by day.
     *
     * @param filter used to filter the result set.
     */
    sessionDuration(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("sessionDuration");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.SESSION_DURATION, filter);
        });
    }
    /**
     * timeOnPage returns the time spent on pages.
     *
     * @param filter used to filter the result set.
     */
    timeOnPage(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("timeOnPage");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.TIME_ON_PAGE, filter);
        });
    }
    /**
     * utmSource returns the utm sources.
     *
     * @param filter used to filter the result set.
     */
    utmSource(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("utmSource");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.UTM_SOURCE, filter);
        });
    }
    /**
     * utmMedium returns the utm medium.
     *
     * @param filter used to filter the result set.
     */
    utmMedium(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("utmMedium");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.UTM_MEDIUM, filter);
        });
    }
    /**
     * utmCampaign returns the utm campaigns.
     *
     * @param filter used to filter the result set.
     */
    utmCampaign(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("utmCampaign");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.UTM_CAMPAIGN, filter);
        });
    }
    /**
     * utmContent returns the utm content.
     *
     * @param filter used to filter the result set.
     */
    utmContent(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("utmContent");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.UTM_CONTENT, filter);
        });
    }
    /**
     * utmTerm returns the utm term.
     *
     * @param filter used to filter the result set.
     */
    utmTerm(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("utmTerm");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.UTM_TERM, filter);
        });
    }
    /**
     * totalVisitors returns the total visitor statistics.
     *
     * @param filter used to filter the result set.
     */
    totalVisitors(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("totalVisitors");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.TOTAL_VISITORS, filter);
        });
    }
    /**
     * visitors returns the visitor statistics grouped by day.
     *
     * @param filter used to filter the result set.
     */
    visitors(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("visitors");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.VISITORS, filter);
        });
    }
    /**
     * entryPages returns the entry page statistics grouped by page.
     *
     * @param filter used to filter the result set.
     */
    entryPages(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("entryPages");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.ENTRY_PAGES, filter);
        });
    }
    /**
     * exitPages returns the exit page statistics grouped by page.
     *
     * @param filter used to filter the result set.
     */
    exitPages(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("exitPages");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.EXIT_PAGES, filter);
        });
    }
    /**
     * pages returns the page statistics grouped by page.
     *
     * @param filter used to filter the result set.
     */
    pages(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("pages");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.PAGES, filter);
        });
    }
    /**
     * conversionGoals returns all conversion goals.
     *
     * @param filter used to filter the result set.
     */
    conversionGoals(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("conversionGoals");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.CONVERSION_GOALS, filter);
        });
    }
    /**
     * events returns all events.
     *
     * @param filter used to filter the result set.
     */
    events(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("events");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.EVENTS, filter);
        });
    }
    /**
     * eventMetadata returns the metadata for a single event.
     * The event name and metadata key must be set in the filter, or otherwise no results will be returned.
     *
     * @param filter used to filter the result set.
     */
    eventMetadata(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("eventMetadata");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.EVENT_METADATA, filter);
        });
    }
    /**
     * listEvents returns a list of all events including metadata.
     *
     * @param filter used to filter the result set.
     */
    listEvents(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("listEvents");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.LIST_EVENTS, filter);
        });
    }
    /**
     * eventPages returns all pages an event has been triggered on.
     * The event name must be set in the filter, or otherwise no results will be returned.
     *
     * @param filter used to filter the result set.
     */
    eventPages(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("eventPages");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.EVENTS_PAGES, filter);
        });
    }
    /**
     * growth returns the growth rates for visitors, bounces, ...
     *
     * @param filter used to filter the result set.
     */
    growth(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("growth");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.GROWTH_RATE, filter);
        });
    }
    /**
     * activeVisitors returns the active visitors and what pages they're on.
     *
     * @param filter used to filter the result set.
     */
    activeVisitors(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("activeVisitors");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.ACTIVE_VISITORS, filter);
        });
    }
    /**
     * timeOfDay returns the number of unique visitors grouped by time of day.
     *
     * @param filter used to filter the result set.
     */
    timeOfDay(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("timeOfDay");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.TIME_OF_DAY, filter);
        });
    }
    /**
     * languages returns language statistics.
     *
     * @param filter used to filter the result set.
     */
    languages(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("languages");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.LANGUAGE, filter);
        });
    }
    /**
     * referrer returns referrer statistics.
     *
     * @param filter used to filter the result set.
     */
    referrer(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("referrer");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.REFERRER, filter);
        });
    }
    /**
     * os returns operating system statistics.
     *
     * @param filter used to filter the result set.
     */
    os(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("os");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.OS, filter);
        });
    }
    /**
     * osVersions returns operating system version statistics.
     *
     * @param filter used to filter the result set.
     */
    osVersions(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("osVersions");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.OS_VERSION, filter);
        });
    }
    /**
     * browser returns browser statistics.
     *
     * @param filter used to filter the result set.
     */
    browser(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("browser");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.BROWSER, filter);
        });
    }
    /**
     * browserVersions returns browser version statistics.
     *
     * @param filter used to filter the result set.
     */
    browserVersions(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("browserVersions");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.BROWSER_VERSION, filter);
        });
    }
    /**
     * country returns country statistics.
     *
     * @param filter used to filter the result set.
     */
    country(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("country");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.COUNTRY, filter);
        });
    }
    /**
     * city returns city statistics.
     *
     * @param filter used to filter the result set.
     */
    city(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("city");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.CITY, filter);
        });
    }
    /**
     * platform returns the platforms used by visitors.
     *
     * @param filter used to filter the result set.
     */
    platform(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("platform");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.PLATFORM, filter);
        });
    }
    /**
     * screen returns the screen classes used by visitors.
     *
     * @param filter used to filter the result set.
     */
    screen(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("screen");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.SCREEN, filter);
        });
    }
    /**
     * screen returns the screen classes used by visitors.
     *
     * @param filter used to filter the result set.
     */
    tagKeys(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("tag_keys");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.TAG_KEYS, filter);
        });
    }
    /**
     * screen returns the screen classes used by visitors.
     *
     * @param filter used to filter the result set.
     */
    tags(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("tags");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.TAG_DETAILS, filter);
        });
    }
    /**
     * keywords returns the Google keywords, rank, and CTR.
     *
     * @param filter used to filter the result set.
     */
    keywords(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            this.accessModeCheck("keywords");
            return yield this.performFilteredGet(constants_1.PirschEndpoint.KEYWORDS, filter);
        });
    }
    performPost(path, data, retry = true) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.post(this.generateUrl(path), data, {
                    headers: {
                        "Content-Type": "application/json",
                        Authorization: `Bearer ${this.accessToken}`,
                    },
                });
                return;
            }
            catch (error) {
                const exception = yield this.toApiError(error);
                if (this.accessMode === "oauth" && exception.code === 401 && retry) {
                    yield this.refreshToken();
                    return this.performPost(path, data, false);
                }
                throw exception;
            }
        });
    }
    performGet(path, parameters = {}, retry = true) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this.accessToken && retry) {
                    yield this.refreshToken();
                }
                const data = yield this.get(this.generateUrl(path), {
                    headers: {
                        "Content-Type": "application/json",
                        Authorization: `Bearer ${this.accessToken}`,
                    },
                    parameters,
                });
                return data;
            }
            catch (error) {
                const exception = yield this.toApiError(error);
                if (this.accessMode === "oauth" && exception.code === 401 && retry) {
                    yield this.refreshToken();
                    return this.performGet(path, parameters, false);
                }
                throw exception;
            }
        });
    }
    performFilteredGet(url, filter) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.performGet(url, this.getFilterParams(filter));
        });
    }
    getFilterParams(filter) {
        const params = {
            id: filter.id,
            from: filter.from,
            to: filter.to,
            start: filter.start,
            scale: filter.scale,
            tz: filter.tz,
            path: filter.path,
            pattern: filter.pattern,
            entry_path: filter.entry_path,
            exit_path: filter.exit_path,
            event: filter.event,
            event_meta_key: filter.event_meta_key,
            language: filter.language,
            country: filter.country,
            city: filter.city,
            referrer: filter.referrer,
            referrer_name: filter.referrer_name,
            os: filter.os,
            browser: filter.browser,
            platform: filter.platform,
            screen_class: filter.screen_class,
            utm_source: filter.utm_source,
            utm_medium: filter.utm_medium,
            utm_campaign: filter.utm_campaign,
            utm_content: filter.utm_content,
            utm_term: filter.utm_term,
            tag: filter.tag,
            custom_metric_key: filter.custom_metric_key,
            custom_metric_type: filter.custom_metric_type,
            search: filter.search,
            offset: filter.offset,
            limit: filter.limit,
            sort: filter.sort,
            direction: filter.direction,
            include_avg_time_on_page: filter.include_avg_time_on_page
        };
        if (filter.event_meta) {
            for (const [key, value] of Object.entries(filter.event_meta)) {
                params[`meta_${key}`] = value;
            }
        }
        if (filter.tags) {
            for (const [key, value] of Object.entries(filter.tags)) {
                params[`tag_${key}`] = value;
            }
        }
        return params;
    }
    refreshToken() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.accessMode === "access-token") {
                return;
            }
            try {
                const result = yield this.post(this.generateUrl(constants_1.PirschEndpoint.AUTHENTICATION), {
                    client_id: this.clientId,
                    client_secret: this.clientSecret,
                });
                this.accessToken = result.access_token;
                return;
            }
            catch (error) {
                this.accessToken = "";
                const exception = yield this.toApiError(error);
                return exception;
            }
        });
    }
    generateUrl(path) {
        return "/" + [this.endpoint, this.version, path].join("/");
    }
    accessModeCheck(methodName) {
        if (this.accessMode === "access-token") {
            throw new common_1.PirschInvalidAccessModeApiError(methodName);
        }
    }
}
exports.PirschCoreClient = PirschCoreClient;
